/**
 * gclc.system:net.bigeon.gclc.system.ExecSystemCommand.java
 * Created on: Jun 20, 2016
 */
package net.bigeon.gclc.system;

/*-
 * #%L
 * GCLC system command
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import net.bigeon.gclc.command.Command;
import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.CommandRunExceptionType;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;

/** A command that will execute system commands.
 * <p>
 * Note that this class may induce security issues in the code as it can execute
 * arbitrary system commands. It is recommended that it be used only in private
 * applications or application used by aware parties.
 *
 * @author Emmanuel Bigeon */
public class ExecSystemCommand extends Command {

    /** Error message for the manager closed. */
    private static final String MANAGER_WAS_CLOSED   = "manager was closed";
    /** The command default name */
    private static final String COMMAND_DEFAULT_NAME = "exec";                //$NON-NLS-1$
    /** The end of line separator */
    private static final String EOL                  = System.lineSeparator();

    /** Create the command with the name 'exec'. */
    public ExecSystemCommand() {
        super(COMMAND_DEFAULT_NAME);
    }

    /** Create the command with a custom name.
     * 
     * @param name the name of the command (the input from the manager that should
     *        trigger this command) */
    public ExecSystemCommand(final String name) {
        super(name);
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#execute(fr.bigeon.gclc.manager.
     * ConsoleOutput, fr.bigeon.gclc.manager.ConsoleInput, java.lang.String[]) */
    @Override
    public void execute(final ConsoleOutput out, final ConsoleInput in,
            final String... args) throws CommandRunException {
        Process proc;
        try {
            proc = Runtime.getRuntime().exec(args);
        } catch (final IOException e2) {
            throw new CommandRunException("Unable to run process", e2);
        }
        // Stream forwarding to the application's console
        // This is started in a parallel thread.
        final Thread th = ForwardingRunnable.connect(proc.getInputStream(), out,
                "Command output");
        // Set the empty prompt for the processes promptings.
        in.setPrompt(""); //$NON-NLS-1$
        // Forward console input to the process, whether the process is waiting or not
        // for input.
        final OutputStream os = proc.getOutputStream();
        try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(os))) {
            while (th.isAlive()) {
                final String user = in.prompt();
                // Avoid interruption being sent to process.
                if (!user.isEmpty()) {
                    writer.write(user + EOL);
                }
            }
        } catch (final IOException e1) {
            throw new CommandRunException(CommandRunExceptionType.INTERACTION,
                    MANAGER_WAS_CLOSED, e1); // $NON-NLS-1$
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.ICommand#tip() */
    @Override
    public String tip() {
        return "Execute a system command"; //$NON-NLS-1$
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usageDetail() */
    @Override
    protected String usageDetail() {
        return "  The system command is a system dependend command like sh on linux or" + //$NON-NLS-1$
                System.lineSeparator() + "powershell on windows." + //$NON-NLS-1$
                System.lineSeparator() + System.lineSeparator()
                + "  As an example if you give \"cat\", \"/etc/hostname\" as arguments, on a linux" //$NON-NLS-1$
                + System.lineSeparator() + "system, you would get the computer name." + //$NON-NLS-1$
                System.lineSeparator();
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.command.Command#usagePattern() */
    @Override
    protected String usagePattern() {
        return "  CMD <system command>"; //$NON-NLS-1$
    }
}
